package ch.psi.daq.queryrest.response.json;

import java.io.OutputStream;
import java.util.List;
import java.util.Map.Entry;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.tuple.Triple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;

import com.hazelcast.util.collection.ArrayUtils;

import ch.psi.daq.domain.FieldNames;
import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueries;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.operation.Compression;
import ch.psi.daq.domain.query.operation.QueryField;
import ch.psi.daq.domain.query.response.ResponseFormat;
import ch.psi.daq.queryrest.query.QueryManager;
import ch.psi.daq.queryrest.response.AbstractHTTPResponse;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;

public class JSONHTTPResponse extends AbstractHTTPResponse {
   private static final Logger LOGGER = LoggerFactory.getLogger(JSONHTTPResponse.class);

   public static final String FORMAT = "json";
   public static final String CONTENT_TYPE = MediaType.APPLICATION_JSON_VALUE;

   public JSONHTTPResponse() {
      super(ResponseFormat.JSON);
   }

   public JSONHTTPResponse(Compression compression) {
      this();
      setCompression(compression);
   }

   @Override
   public void respond(ApplicationContext context, DAQQueries queries, HttpServletResponse response) throws Exception {
      OutputStream out = handleCompressionAndResponseHeaders(response, CONTENT_TYPE);

      boolean hasMapping = JSONHTTPResponse.validateQueries(queries);

      try {
         LOGGER.debug("Executing query '{}'", queries);

         QueryManager queryManager = context.getBean(QueryManager.class);
         ResponseStreamWriter streamWriter;
         if (hasMapping) {
            streamWriter = context.getBean(JSONTableResponseStreamWriter.class);
         } else {
            streamWriter = context.getBean(JSONResponseStreamWriter.class);
         }

         // execute query
         List<Entry<DAQQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>>> result =
               queryManager.getEvents(queries);
         // write the response back to the client using java 8 streams
         streamWriter.respond(result, out, this);
      } catch (Exception e) {
         LOGGER.error("Failed to execute query '{}'.", queries, e);
         throw e;
      }
   }


   public static boolean validateQueries(DAQQueries queries) {
      boolean hasMapping = false;

      for (DAQQueryElement query : queries) {
         if (query.getMapping() != null) {
            hasMapping = true;

            if (!ArrayUtils.contains(query.getColumns(), FieldNames.FIELD_GLOBAL_TIME)) {
               query.addField(QueryField.globalMillis);
            }
         }
      }
      
      return hasMapping;
   }
}
