package ch.psi.daq.queryrest.response.msgpack;

import java.io.OutputStream;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.servlet.ServletResponse;

import org.apache.commons.lang3.tuple.Triple;
import org.msgpack.jackson.dataformat.MessagePackFactory;
import org.springframework.context.ApplicationContext;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.psi.daq.domain.json.ChannelName;
import ch.psi.daq.domain.query.DAQQueryElement;
import ch.psi.daq.domain.query.backend.BackendQuery;
import ch.psi.daq.domain.query.operation.Aggregation;
import ch.psi.daq.domain.query.response.Response;
import ch.psi.daq.queryrest.config.QueryRestConfig;
import ch.psi.daq.queryrest.response.ResponseStreamWriter;
import ch.psi.daq.queryrest.response.json.JSONTableResponseStreamWriter;

/**
 * Takes a Java 8 stream and writes it to the output stream provided by the {@link ServletResponse}
 * of the current request.
 */
public class MsgPackTableResponseStreamWriter implements ResponseStreamWriter {

   @Resource
   private ApplicationContext context;

   @Resource
   private MessagePackFactory msgPackFactory;

   @Resource
   private ObjectMapper mapper;

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_AGGREGATIONS)
   private Set<Aggregation> defaultResponseAggregations;

   private Set<String> defaultResponseAggregationsStr;

   @PostConstruct
   public void afterPropertiesSet() {
      defaultResponseAggregationsStr =
            defaultResponseAggregations.stream().map(Aggregation::name)
                  .collect(Collectors.toCollection(LinkedHashSet::new));
   }

   @Override
   public void respond(final List<Entry<DAQQueryElement, Stream<Triple<BackendQuery, ChannelName, ?>>>> results,
         final OutputStream out, final Response response) throws Exception {     
      JSONTableResponseStreamWriter.respond(context, msgPackFactory, mapper, defaultResponseAggregationsStr, results, out, response);
   }
}
