package ch.psi.daq.test.queryrest.config;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurationSupport;

import ch.psi.daq.archiverappliance.config.ArchiverApplianceConfig;
import ch.psi.daq.cassandra.config.CassandraConfig;
import ch.psi.daq.cassandra.reader.CassandraReader;
import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.backend.BackendAccess;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.events.ChannelEvent;
import ch.psi.daq.domain.query.processor.QueryProcessor;
import ch.psi.daq.domain.reader.DataReader;
import ch.psi.daq.domain.reader.StreamEventReader;
import ch.psi.daq.filestorage.config.FileStorageConfig;
import ch.psi.daq.query.config.QueryConfig;
import ch.psi.daq.query.processor.QueryProcessorLocal;
import ch.psi.daq.test.query.config.LocalQueryTestConfig;
import ch.psi.daq.test.queryrest.query.DummyArchiverApplianceReader;
import ch.psi.daq.test.queryrest.query.DummyCassandraReader;
import ch.psi.daq.test.queryrest.query.DummyFilestorageReader;

@Configuration
@ComponentScan
@EnableWebMvc
@PropertySources(value = {
      @PropertySource(value = {"classpath:queryrest-test.properties"})
})
public class DaqWebMvcConfig extends WebMvcConfigurationSupport {

   // ensure that properties in dispatcher.properties are loaded first and then overwritten by the
   // properties in dispatcher-test.properties
   @Import(value = {LocalQueryTestConfig.class})
   static class InnerConfiguration {
   }

   @Resource(name = DomainConfig.BEAN_NAME_BACKEND_ACCESS)
   private BackendAccess backendAccess;

   @PostConstruct
   public void afterPropertiesSet() {
      backendAccess.addStreamEventReaderSupplier(Backend.SF_DATABUFFER, () -> cassandraReader());
      backendAccess.addChannelInfoReaderSupplier(Backend.SF_DATABUFFER, () -> cassandraReader());
      
      backendAccess.addStreamEventReaderSupplier(Backend.SF_IMAGESTORAGE, () -> filestorageReader());
      backendAccess.addChannelInfoReaderSupplier(Backend.SF_IMAGESTORAGE, () -> filestorageReader());

      backendAccess.addDataReaderSupplier(Backend.SF_ARCHIVERAPPLIANCE, () -> archiverApplianceReader());
   }

   // make sure we use a local QueryProcessor even for distributed calls -> no Hazelcast needs to be started
   @Bean(name = QueryConfig.BEAN_NAME_QUERY_PROCESSOR_DISTRIBUTED)
   @Lazy
   public QueryProcessor distributedQueryProcessor() {
      return new QueryProcessorLocal();
   }

   @Bean(name = CassandraConfig.BEAN_NAME_CASSANDRA_READER)
   @Lazy
   public CassandraReader cassandraReader() {
      return new DummyCassandraReader();
   }
   
   @Bean(name = FileStorageConfig.BEAN_NAME_FILESTORAGE_READER)
   @Lazy
   public StreamEventReader<ChannelEvent> filestorageReader() {
      return new DummyFilestorageReader();
   }

   @Bean(name = ArchiverApplianceConfig.BEAN_NAME_ARCHIVER_APPLIANCE_READER)
   @Lazy
   public DataReader archiverApplianceReader() {
      return new DummyArchiverApplianceReader();
   }
}
